# yaranullin/board.py

#
# Copyright (c) 2010 Marco Scopesi <marco.scopesi@gmail.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#


class Board(object):
    """The game board.

    This object retains a grid[][] containing a reference to all the
    initiatives and some methods to add, remove and move initiatives in the
    grid. On his own, this is a textual object.
    """

    def __init__(self, size, game=None, renderingObject=None):
        self.game = game
        self.initiatives = []
        self.pawns = {}
        self.current_pawn = None
        self.size = size
        self.renderingObject = renderingObject
        nx, ny = self.size
        self.grid = [[list() for y in range(ny)] for x in range(nx)]

    def add_pawn(self, name, initiative, pos, *groups):
        if name in self.pawns:
            print "A pawn with name %s already exists" % name
            return
        if self.renderingObject:
            from pawn import PawnSprite
            pawn = PawnSprite(name, initiative, pos, *groups)
        else:
            from pawn import Pawn
            pawn = Pawn(name, initiative, pos)
        x, y = pos
        self.pawns[name] = pawn
        self.grid[x][y].append(pawn)
        self.initiatives.append(pawn)
        self.initiatives.sort(key=lambda pawn: pawn.initiative,\
            reverse=True)
        if self.current_pawn == None:
            self.current_pawn = pawn
        print "%s added with initiative %d" % (pawn.name, pawn.initiative)
        return True

    def remove_pawn(self, pawn):
        if self.renderingObject:
            from pawn import PawnSprite as Pawn
        else:
            from pawn import Pawn
        if not isinstance(pawn, Pawn):
            return
        try:
            [x, y] = pawn.pos
            del self.pawns[pawn.name]
            self.grid[x][y].remove(pawn)
            self.initiatives.remove(pawn)
            if len(self.initiatives) == 0:
                self.current_pawn = None
            elif pawn == self.current_pawn:
                self.next_pawn()
            print "%s deleted" % pawn.name
            return True
        except:
            print "Cannot remove pawn %s" % pawn.name
            return False

    def next_pawn(self):
        current_pawn = self.current_pawn
        current_pawn_index = self.initiatives.index(current_pawn)
        new_pawn_index = current_pawn_index + 1
        try:
            self.current_pawn = self.initiatives[new_pawn_index]
        except IndexError:
            self.current_pawn = self.initiatives[0]

    def move_pawn(self, (new_x, new_y), pawn=None):
        if self.current_pawn == None:
            print "No pawn on the board"
            return
        if pawn == None:
            pawn = self.current_pawn
        old_x = pawn.pos[0]
        old_y = pawn.pos[1]
        self.grid[old_x][old_y].remove(pawn)
        self.grid[new_x][new_y].append(pawn)
        pawn.pos = (new_x, new_y)
        return True
