# yaranullin/game.py

#
# Copyright (c) 2010 Marco Scopesi <marco.scopesi@gmail.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
from xml.etree.ElementTree import ElementTree, Element, SubElement, parse

import pgu_gui as gui
import timer
from PodSixNet.Connection import connection, ConnectionListener
from conf import conf
from drawingarea import RenderingArea


class Game(gui.App, ConnectionListener):
    """This is the engine of Yaranullin.

    It contanins a reference to the state of the game and provides methods to
    load and save a game state. It provides functions to add, remove and move
    pawns and to initialize the main board of the game. Moreover it contains
    the main loop function (run).
    """

    def __init__(self):
        gui.App.__init__(self)
        self.conf = conf
        self.board = None
        self.board_img_name = None
        self.board_real_width = None
        self.pawns = {}
        self.initiatives = []
        self.current_pawn = None

        host, port = conf['network-host'], conf['network-port']
        self.Connect((host, port))
        print "Trying to connect to Server"

    def Network(self, data):
        print 'network data:', data

    def Network_connected(self, data):
        print "connected to the server"

    def Network_error(self, data):
        print "error:", data['error'][1]

    def Network_disconnected(self, data):
        print "disconnected from the server"

    def init(self):
        w, h = conf['screen-width'], conf['screen-height']
        self.rendering_area = RenderingArea(width=w, height=h)
        gui.App.init(self, self.rendering_area)

    def run(self, FPS=20):
        connection.Send({'action': 'game', 'message': 'A new game is\
            starting...'})
        self.init()
        try:
            self.open_board(conf['main-yr_auto_save'])
        except:
            pass
        clock = timer.Clock()
        while not self._quit:
            self.loop()
            clock.tick(FPS)
            connection.Pump()
            self.Pump()
        self.save_board(conf['main-yr_auto_save'])
        connection.Send({'action': 'game', 'message': 'A game just ended...'})
        connection.Pump()
        self.Pump()

    def set_board(self, img_name, real_w):
        self.rendering_area.set_board(img_name, real_w)
        self.board = self.rendering_area.board
        self.pawns = self.board.pawns
        self.initiatives = self.board.initiatives
        self.current_pawn = self.board.current_pawn
        self.board_img_name = img_name
        self.board_real_width = real_w

    # Logic related functions
    def add_pawn(self, pname, initiative, pos, fname):
        self.board.add_pawn(pname, initiative, pos)
        pawn = self.pawns[pname]
        pawn.img_name = fname
        self.rendering_area.sprites['pawns'].add(pawn)
        self.current_pawn = self.board.current_pawn

    def remove_pawns(self, pawns):
        for pawn in pawns:
            try:
                self.rendering_area.sprites['pawns'].remove(pawn)
                self.board.remove_pawn(pawn)
            except:
                print "Cannot remove all pawns"
        self.current_pawn = self.board.current_pawn

    def next_pawn(self):
        self.board.next_pawn()
        self.current_pawn = self.board.current_pawn

    def move_pawn(self, (new_x, new_y), pawn=None):
        self.board.move_pawn(self, (new_x, new_y), pawn)

    def save_board(self, f):
        board = Element('board', img_name=self.board_img_name,\
            real_w=str(self.board_real_width),\
                current_pawn_name=self.current_pawn.name)
        for pawn in self.initiatives:
            SubElement(board, 'pawn', name=pawn.name, img_name=pawn.img_name,\
                initiative=str(pawn.initiative), pos=str(pawn.pos))
        tree = ElementTree(board)
        tree.write(f)

    def open_board(self, f):
        tree = parse(f)
        board = tree.getroot()
        img_name = board.attrib['img_name']
        real_w = eval(board.attrib['real_w'])
        current_pawn_name = board.attrib['current_pawn_name']
        self.set_board(img_name, real_w)
        for pawn in board:
            name = pawn.attrib['name']
            img_name = pawn.attrib['img_name']
            initiative = eval(pawn.attrib['initiative'])
            pos = eval(pawn.attrib['pos'])
            self.add_pawn(name, initiative, pos, img_name)
        self.board.current_pawn = self.pawns[current_pawn_name]
