# yaranullin/pawn.py

#
# Copyright (c) 2010 Marco Scopesi <marco.scopesi@gmail.com>
# Copyright (c) 2010 Simone Basso <simone.basso@gmail.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
from pygame.sprite import Sprite
from pygame.transform import smoothscale

from toolkit import load_image


class Pawn(object):
    """A PG, an NPG or a monster.

    """

    def __init__(self, name, initiative, pos):
        self.name = name
        self.pos = pos
        self.initiative = initiative


class PawnSprite(Pawn, Sprite):
    """An extended Pawn class to hold for graphic purpouses.

    When settin the property img_name, this class will autoload the image and
    save its initial value. The property image cannot be set directly.
    """

    def __init__(self, name, initiative, pos, *groups):
        Sprite.__init__(self, *groups)
        Pawn.__init__(self, name, initiative, pos)
        self.__img_name = None
        self.__intial_image = None
        self.__image = None
        self.rect = None

    @property
    def image(self):
        return self.__image

    @property
    def img_name(self):
        return self.__img_name

    @img_name.setter
    def img_name(self, value):
        try:
            img, rect = load_image(value)
            self.__initial_image = img
            self.__image = img
            self.rect = rect
            self.__img_name = value
        except:
            pass

    def update(self, rendering_area):
        cell_rect = rendering_area.get_rect_at_pos(self.pos)
        self.rect.width = cell_rect.width - 2 # Double of the grid line width
        self.rect.height = cell_rect.height - 2
        self.rect.center = cell_rect.center
        self.__image = smoothscale(self.__initial_image, (self.rect.width,\
            self.rect.height))
